@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2014 Canonical Ltd.
+# Copyright (C) 2013-2015 Canonical Ltd.
 # Author: Barry Warsaw <barry@ubuntu.com>
 
 # This program is free software: you can redistribute it and/or modify
@@ -67,10 +67,14 @@ class Image(Bag):
     def __ne__(self, other):
         return not self.__eq__(other)
 
-    def __repr__(self):
+    def __repr__(self): # pragma: no cover
         return '<Image: {}>'.format(COMMASPACE.join(sorted(
             key for key in self.__dict__ if not key.startswith('_'))))
 
     @property
     def phased_percentage(self):
         return self.__untranslated__.get('phased-percentage', 100)
+
+    @property
+    def version_detail(self):
+        return self.__untranslated__.get('version_detail', '')
