@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2014 Canonical Ltd.
+# Copyright (C) 2013-2015 Canonical Ltd.
 # Author: Barry Warsaw <barry@ubuntu.com>
 
 # This program is free software: you can redistribute it and/or modify
@@ -16,8 +16,10 @@
 """Test asynchronous downloads."""
 
 __all__ = [
+    'TestCURL',
+    'TestDownload',
     'TestDownloadBigFiles',
-    'TestDownloads',
+    'TestDownloadManagerFactory',
     'TestDuplicateDownloads',
     'TestGSMDownloads',
     'TestHTTPSDownloads',
@@ -29,41 +31,47 @@ __all__ = [
 
 
 import os
-import sys
 import random
 import unittest
 
 from contextlib import ExitStack
-from datetime import datetime, timedelta
-from gi.repository import GLib
+from dbus.exceptions import DBusException
 from hashlib import sha256
 from systemimage.config import Configuration, config
+from systemimage.curl import CurlDownloadManager
 from systemimage.download import (
-    Canceled, DBusDownloadManager, DuplicateDestinationError, Record)
+    Canceled, DuplicateDestinationError, Record, get_download_manager)
 from systemimage.helpers import temporary_directory
 from systemimage.settings import Settings
+from systemimage.testing.controller import USING_PYCURL
 from systemimage.testing.helpers import (
-    configuration, data_path, make_http_server, write_bytes)
+    configuration, data_path, make_http_server, reset_envar, write_bytes)
 from systemimage.testing.nose import SystemImagePlugin
+from systemimage.udm import DOWNLOADER_INTERFACE, UDMDownloadManager
 from unittest.mock import patch
 from urllib.parse import urljoin
 
+if USING_PYCURL:
+    import pycurl
+
 
 def _http_pathify(downloads):
     return [
-        (urljoin(config.service.http_base, url),
+        (urljoin(config.http_base, url),
          os.path.join(config.tempdir, filename)
         ) for url, filename in downloads]
 
 
 def _https_pathify(downloads):
     return [
-        (urljoin(config.service.https_base, url),
+        (urljoin(config.https_base, url),
          os.path.join(config.tempdir, filename)
         ) for url, filename in downloads]
 
 
-class TestDownloads(unittest.TestCase):
+class TestDownload(unittest.TestCase):
+    """Base class for testing the PyCURL and udm downloaders."""
+
     def setUp(self):
         super().setUp()
         self._resources = ExitStack()
@@ -80,12 +88,15 @@ class TestDownloads(unittest.TestCase):
         self._resources.close()
         super().tearDown()
 
+    def _downloader(self, *args):
+        return get_download_manager(*args)
+
     @configuration
     def test_good_path(self):
         # Download a bunch of files that exist.  No callback.
-        DBusDownloadManager().get_files(_http_pathify([
-            ('channels_01.json', 'channels.json'),
-            ('index_01.json', 'index.json'),
+        self._downloader().get_files(_http_pathify([
+            ('channel.channels_05.json', 'channels.json'),
+            ('download.index_01.json', 'index.json'),
             ]))
         self.assertEqual(
             set(os.listdir(config.tempdir)),
@@ -94,18 +105,19 @@ class TestDownloads(unittest.TestCase):
     @configuration
     def test_empty_download(self):
         # Empty download set completes successfully.  LP: #1245597.
-        DBusDownloadManager().get_files([])
+        self._downloader().get_files([])
         # No TimeoutError is raised.
 
     @configuration
     def test_user_agent(self):
         # The User-Agent request header contains the build number.
         version = random.randint(0, 99)
-        with open(config.system.build_file, 'w', encoding='utf-8') as fp:
-            print(version, file=fp)
+        config.build_number = version
+        config.device = 'geddyboard'
+        config.channel = 'devel-trio'
         # Download a magic path which the server will interpret to return us
         # the User-Agent header value.
-        DBusDownloadManager().get_files(_http_pathify([
+        self._downloader().get_files(_http_pathify([
             ('user-agent.txt', 'user-agent.txt'),
             ]))
         path = os.path.join(config.tempdir, 'user-agent.txt')
@@ -113,7 +125,9 @@ class TestDownloads(unittest.TestCase):
             user_agent = fp.read()
         self.assertEqual(
             user_agent,
-            'Ubuntu System Image Upgrade Client; Build {}'.format(version))
+            'Ubuntu System Image Upgrade Client: '
+            'device=geddyboard;channel=devel-trio;build={}'.format(
+                version))
 
     @configuration
     def test_download_with_callback(self):
@@ -124,10 +138,10 @@ class TestDownloads(unittest.TestCase):
             nonlocal received_bytes, total_bytes
             received_bytes = received
             total_bytes = total
-        downloader = DBusDownloadManager(callback)
+        downloader = self._downloader(callback)
         downloader.get_files(_http_pathify([
-            ('channels_01.json', 'channels.json'),
-            ('index_01.json', 'index.json'),
+            ('channel.channels_05.json', 'channels.json'),
+            ('download.index_01.json', 'index.json'),
             ]))
         self.assertEqual(
             set(os.listdir(config.tempdir)),
@@ -144,30 +158,18 @@ class TestDownloads(unittest.TestCase):
         def capture(message):
             nonlocal exception
             exception = message
-        downloader = DBusDownloadManager(callback)
+        downloader = self._downloader(callback)
         with patch('systemimage.download.log.exception', capture):
             downloader.get_files(_http_pathify([
-                ('channels_01.json', 'channels.json'),
+                ('channel.channels_05.json', 'channels.json'),
                 ]))
         # The exception got logged.
         self.assertEqual(exception, 'Exception in progress callback')
         # The file still got downloaded.
         self.assertEqual(os.listdir(config.tempdir), ['channels.json'])
 
-    @configuration
-    def test_no_dev_package(self):
-        # system-image-dev contains the systemimage.testing subpackage, but
-        # this is not normally installed on the device.  When it's missing,
-        # the DownloadReactor's _print() debugging method should no-op.
-        #
-        # To test this, we patch systemimage.testing in sys.modules so that an
-        # ImportError is raised when it tries to import it.
-        with patch.dict(sys.modules, {'systemimage.testing.helpers': None}):
-            DBusDownloadManager().get_files(_http_pathify([
-                ('channels_01.json', 'channels.json'),
-                ]))
-        self.assertEqual(os.listdir(config.tempdir), ['channels.json'])
-
+    # This test helps bump the udm-based downloader test coverage to 100%.
+    @unittest.skipIf(USING_PYCURL, 'Test is not relevant for PyCURL')
     @configuration
     def test_timeout(self):
         # If the reactor times out, we get an exception.  We fake the timeout
@@ -176,12 +178,12 @@ class TestDownloads(unittest.TestCase):
         def finish_with_timeout(self, *args, **kws):
             self.timed_out = True
             self.quit()
-        with patch('systemimage.download.DownloadReactor._do_finished',
+        with patch('systemimage.udm.DownloadReactor._do_finished',
                    finish_with_timeout):
             self.assertRaises(
                 TimeoutError,
-                DBusDownloadManager().get_files,
-                _http_pathify([('channels_01.json', 'channels.json')])
+                self._downloader().get_files,
+                _http_pathify([('channel.channels_05.json', 'channels.json')])
                 )
 
 
@@ -200,8 +202,8 @@ class TestHTTPSDownloads(unittest.TestCase):
         with ExitStack() as stack:
             stack.push(make_http_server(
                 self._directory, 8943, 'cert.pem', 'key.pem'))
-            DBusDownloadManager().get_files(_https_pathify([
-                ('channels_01.json', 'channels.json'),
+            get_download_manager().get_files(_https_pathify([
+                ('channel.channels_05.json', 'channels.json'),
                 ]))
             self.assertEqual(
                 set(os.listdir(config.tempdir)),
@@ -223,9 +225,9 @@ class TestHTTPSDownloadsNoSelfSigned(unittest.TestCase):
         with make_http_server(self._directory, 8943, 'cert.pem', 'key.pem'):
             self.assertRaises(
                 FileNotFoundError,
-                DBusDownloadManager().get_files,
+                get_download_manager().get_files,
                 _https_pathify([
-                    ('channels_01.json', 'channels.json'),
+                    ('channel.channels_05.json', 'channels.json'),
                     ]))
 
     @configuration
@@ -238,9 +240,9 @@ class TestHTTPSDownloadsNoSelfSigned(unittest.TestCase):
             stack.push(make_http_server(self._directory, 8943))
             self.assertRaises(
                 FileNotFoundError,
-                DBusDownloadManager().get_files,
+                get_download_manager().get_files,
                 _https_pathify([
-                    ('channels_01.json', 'channels.json'),
+                    ('channel.channels_05.json', 'channels.json'),
                     ]))
 
 
@@ -261,9 +263,9 @@ class TestHTTPSDownloadsExpired(unittest.TestCase):
                 self._directory, 8943, 'expired_cert.pem', 'expired_key.pem'))
             self.assertRaises(
                 FileNotFoundError,
-                DBusDownloadManager().get_files,
+                get_download_manager().get_files,
                 _https_pathify([
-                    ('channels_01.json', 'channels.json'),
+                    ('channel.channels_05.json', 'channels.json'),
                     ]))
 
 
@@ -284,12 +286,16 @@ class TestHTTPSDownloadsNasty(unittest.TestCase):
                 self._directory, 8943, 'nasty_cert.pem', 'nasty_key.pem'))
             self.assertRaises(
                 FileNotFoundError,
-                DBusDownloadManager().get_files,
+                get_download_manager().get_files,
                 _https_pathify([
-                    ('channels_01.json', 'channels.json'),
+                    ('channel.channels_05.json', 'channels.json'),
                     ]))
 
 
+# These tests don't strictly improve coverage for the udm-based downloader,
+# but they are still useful to keep because they test a implicit code path.
+# These can be removed once GSM-testing is pulled into s-i via LP: #1388886.
+@unittest.skipIf(USING_PYCURL, 'Test is not relevant for PyCURL')
 class TestGSMDownloads(unittest.TestCase):
     def setUp(self):
         super().setUp()
@@ -311,9 +317,9 @@ class TestGSMDownloads(unittest.TestCase):
             directory = os.path.dirname(data_path('__init__.py'))
             self._resources.push(make_http_server(directory, 8980))
             # Patch the GSM setting method to capture what actually happens.
-            self._original = getattr(DBusDownloadManager, '_set_gsm')
+            self._original = getattr(UDMDownloadManager, '_set_gsm')
             self._resources.enter_context(patch(
-                'systemimage.download.DBusDownloadManager._set_gsm', set_gsm))
+                'systemimage.udm.UDMDownloadManager._set_gsm', set_gsm))
             self._resources.callback(setattr, self, '_original', None)
         except:
             self._resources.close()
@@ -324,42 +330,44 @@ class TestGSMDownloads(unittest.TestCase):
         super().tearDown()
 
     @configuration
-    def test_manual_downloads_gsm_allowed(self, ini_file):
+    def test_manual_downloads_gsm_allowed(self, config_d):
         # When auto_download is 0, manual downloads are enabled so assuming
         # the user knows what they're doing, GSM downloads are allowed.
-        config = Configuration(ini_file)
+        config = Configuration(config_d)
         Settings(config).set('auto_download', '0')
-        DBusDownloadManager().get_files(_http_pathify([
-            ('channels_01.json', 'channels.json')
+        get_download_manager().get_files(_http_pathify([
+            ('channel.channels_05.json', 'channels.json')
             ]))
         self.assertTrue(self._gsm_set_flag)
         self.assertTrue(self._gsm_get_flag)
 
     @configuration
-    def test_wifi_downloads_gsm_disallowed(self, ini_file):
+    def test_wifi_downloads_gsm_disallowed(self, config_d):
         # Obviously GSM downloads are not allowed when downloading
         # automatically on wifi-only.
-        config = Configuration(ini_file)
+        config = Configuration(config_d)
         Settings(config).set('auto_download', '1')
-        DBusDownloadManager().get_files(_http_pathify([
-            ('channels_01.json', 'channels.json')
+        get_download_manager().get_files(_http_pathify([
+            ('channel.channels_05.json', 'channels.json')
             ]))
         self.assertFalse(self._gsm_set_flag)
         self.assertFalse(self._gsm_get_flag)
 
     @configuration
-    def test_always_downloads_gsm_allowed(self, ini_file):
+    def test_always_downloads_gsm_allowed(self, config_d):
         # GSM downloads are allowed when always downloading.
-        config = Configuration(ini_file)
+        config = Configuration(config_d)
         Settings(config).set('auto_download', '2')
-        DBusDownloadManager().get_files(_http_pathify([
-            ('channels_01.json', 'channels.json')
+        get_download_manager().get_files(_http_pathify([
+            ('channel.channels_05.json', 'channels.json')
             ]))
         self.assertTrue(self._gsm_set_flag)
         self.assertTrue(self._gsm_get_flag)
 
 
 class TestDownloadBigFiles(unittest.TestCase):
+    # This test helps bump the udm-based downloader test coverage to 100%.
+    @unittest.skipIf(USING_PYCURL, 'Test is not relevant for PyCURL')
     @configuration
     def test_cancel(self):
         # Try to cancel the download of a big file.
@@ -373,12 +381,12 @@ class TestDownloadBigFiles(unittest.TestCase):
             # The download service doesn't provide reliable cancel
             # granularity, so instead, we mock the 'started' signal to
             # immediately cancel the download.
-            downloader = DBusDownloadManager()
+            downloader = get_download_manager()
             def cancel_on_start(self, signal, path, started):
                 if started:
                     downloader.cancel()
             stack.enter_context(patch(
-                'systemimage.download.DownloadReactor._do_started',
+                'systemimage.udm.DownloadReactor._do_started',
                 cancel_on_start))
             self.assertRaises(
                 Canceled, downloader.get_files, _http_pathify([
@@ -407,53 +415,11 @@ class TestDownloadBigFiles(unittest.TestCase):
                 ('missing.txt', 'missing.txt'),
                 ])
             self.assertRaises(FileNotFoundError,
-                              DBusDownloadManager().get_files,
+                              get_download_manager().get_files,
                               downloads)
             # The temporary directory is empty.
             self.assertEqual(os.listdir(config.tempdir), [])
 
-    @configuration
-    def test_download_pause_resume(self):
-        with ExitStack() as stack:
-            serverdir = stack.enter_context(temporary_directory())
-            stack.push(make_http_server(serverdir, 8980))
-            # Create a couple of big files to download.
-            write_bytes(os.path.join(serverdir, 'bigfile_1.dat'), 10)
-            write_bytes(os.path.join(serverdir, 'bigfile_2.dat'), 10)
-            write_bytes(os.path.join(serverdir, 'bigfile_3.dat'), 10)
-            downloads = _http_pathify([
-                ('bigfile_1.dat', 'bigfile_1.dat'),
-                ('bigfile_2.dat', 'bigfile_2.dat'),
-                ('bigfile_3.dat', 'bigfile_3.dat'),
-                ])
-            downloader = DBusDownloadManager()
-            pauses = []
-            def do_paused(self, signal, path, paused):
-                if paused:
-                    pauses.append(datetime.now())
-            resumes = []
-            def do_resumed(self, signal, path, resumed):
-                if resumed:
-                    resumes.append(datetime.now())
-            def pause_on_start(self, signal, path, started):
-                if started:
-                    downloader.pause()
-                    GLib.timeout_add_seconds(3, downloader.resume)
-            stack.enter_context(
-                patch('systemimage.download.DownloadReactor._do_paused',
-                      do_paused))
-            stack.enter_context(
-                patch('systemimage.download.DownloadReactor._do_resumed',
-                      do_resumed))
-            stack.enter_context(
-                patch('systemimage.download.DownloadReactor._do_started',
-                      pause_on_start))
-            downloader.get_files(downloads, pausable=True)
-            self.assertEqual(len(pauses), 1)
-            self.assertEqual(len(resumes), 1)
-            self.assertGreaterEqual(resumes[0] - pauses[0],
-                                    timedelta(seconds=2.5))
-
 
 class TestRecord(unittest.TestCase):
     def test_record(self):
@@ -506,7 +472,7 @@ class TestDuplicateDownloads(unittest.TestCase):
         checksum = sha256(content).hexdigest()
         with open(os.path.join(self._serverdir, 'source.dat'), 'wb') as fp:
             fp.write(content)
-        downloader = DBusDownloadManager()
+        downloader = get_download_manager()
         downloads = []
         for url, dst in _http_pathify([('source.dat', 'local.dat'),
                                        ('source.dat', 'local.dat'),
@@ -525,7 +491,7 @@ class TestDuplicateDownloads(unittest.TestCase):
             fp.write(content)
         with open(os.path.join(self._serverdir, 'source2.dat'), 'wb') as fp:
             fp.write(content)
-        downloader = DBusDownloadManager()
+        downloader = get_download_manager()
         downloads = []
         for url, dst in _http_pathify([('source1.dat', 'local.dat'),
                                        ('source2.dat', 'local.dat'),
@@ -549,8 +515,8 @@ class TestDuplicateDownloads(unittest.TestCase):
         checksum = sha256(content).hexdigest()
         with open(os.path.join(self._serverdir, 'source.dat'), 'wb') as fp:
             fp.write(content)
-        downloader = DBusDownloadManager()
-        url = urljoin(config.service.http_base, 'source.dat')
+        downloader = get_download_manager()
+        url = urljoin(config.http_base, 'source.dat')
         downloads = [
             Record(url, 'local.dat', checksum),
             # Mutate the checksum so they won't match.
@@ -581,8 +547,8 @@ class TestDuplicateDownloads(unittest.TestCase):
         checksum = sha256(content).hexdigest()
         with open(os.path.join(self._serverdir, 'source.dat'), 'wb') as fp:
             fp.write(content)
-        downloader = DBusDownloadManager()
-        url = urljoin(config.service.http_base, 'source.dat')
+        downloader = get_download_manager()
+        url = urljoin(config.http_base, 'source.dat')
         downloads = [
             Record(url, 'local.dat', checksum),
             # Mutate the checksum so they won't match.
@@ -598,3 +564,151 @@ class TestDuplicateDownloads(unittest.TestCase):
             (   'http://localhost:8980/source.dat',
                 'local.dat',
                 '809ecb6ebc8bcefc733f6f2ec44f791abeed6a99edf0cc31519637898aebd52d')])]""")
+
+
+# This class only bumps coverage to 100% for the cURL-based downloader, so it
+# can be skipped when the test suite runs under u-d-m.  Checking the
+# environment variable wouldn't be enough for production (see download.py
+# get_download_manager() for other cases where the downloader is chosen), but
+# it's sufficient for the test suite.  See tox.ini.
+@unittest.skipUnless(USING_PYCURL, 'Test is not relevant for UDM')
+class TestCURL(unittest.TestCase):
+    def setUp(self):
+        super().setUp()
+        self._resources = ExitStack()
+        try:
+            # Start the HTTP server running, vending files out of our test
+            # data directory.
+            directory = os.path.dirname(data_path('__init__.py'))
+            self._resources.push(make_http_server(directory, 8980))
+        except:
+            self._resources.close()
+            raise
+
+    def tearDown(self):
+        self._resources.close()
+        super().tearDown()
+
+    @configuration
+    def test_multi_perform(self):
+        # PyCURL's multi.perform() can return the E_CALL_MULTI_PEFORM status
+        # which tells us to just try again.  This doesn't happen in practice,
+        # but the code path needs coverage.  However, .perform() itself can't
+        # be mocked because pycurl.CurlMulti is a built-in.  Fun.
+        class FakeMulti:
+            def perform(self):
+                return pycurl.E_CALL_MULTI_PERFORM, 2
+        done_once = False
+        class Testable(CurlDownloadManager):
+            def _do_once(self, multi, handles):
+                nonlocal done_once
+                if done_once:
+                    return super()._do_once(multi, handles)
+                else:
+                    done_once = True
+                    return super()._do_once(FakeMulti(), handles)
+        Testable().get_files(_http_pathify([
+            ('channel.channels_05.json', 'channels.json'),
+            ('download.index_01.json', 'index.json'),
+            ]))
+        self.assertTrue(done_once)
+        # The files still get downloaded.
+        self.assertEqual(
+            set(os.listdir(config.tempdir)),
+            set(['channels.json', 'index.json']))
+
+    @configuration
+    def test_multi_fail(self):
+        # PyCURL's multi.perform() can return a failure code (i.e. not E_OK)
+        # which triggers a FileNotFoundError.  It doesn't really matter which
+        # failure code it returns.
+        class FakeMulti:
+            def perform(self):
+                return pycurl.E_READ_ERROR, 2
+        class Testable(CurlDownloadManager):
+            def _do_once(self, multi, handles):
+                return super()._do_once(FakeMulti(), handles)
+        with self.assertRaises(FileNotFoundError) as cm:
+            Testable().get_files(_http_pathify([
+                ('channel.channels_05.json', 'channels.json'),
+                ('download.index_01.json', 'index.json'),
+                ]))
+        # One of the two files will be contained in the error message, but
+        # which one is undefined, although in practice it will be the first
+        # one.
+        self.assertRegex(
+            cm.exception.args[0],
+            'http://localhost:8980/(channel.channels_05|index_01).json')
+
+
+class TestDownloadManagerFactory(unittest.TestCase):
+    """We have a factory for creating the download manager to use."""
+
+    def test_get_downloader_forced_curl(self):
+        # Setting SYSTEMIMAGE_PYCURL envar to 1, yes, or true forces the
+        # PyCURL downloader.
+        with reset_envar('SYSTEMIMAGE_PYCURL'):
+            os.environ['SYSTEMIMAGE_PYCURL'] = '1'
+            self.assertIsInstance(get_download_manager(), CurlDownloadManager)
+        with reset_envar('SYSTEMIMAGE_PYCURL'):
+            os.environ['SYSTEMIMAGE_PYCURL'] = 'tRuE'
+            self.assertIsInstance(get_download_manager(), CurlDownloadManager)
+        with reset_envar('SYSTEMIMAGE_PYCURL'):
+            os.environ['SYSTEMIMAGE_PYCURL'] = 'YES'
+            self.assertIsInstance(get_download_manager(), CurlDownloadManager)
+
+    def test_get_downloader_forced_udm(self):
+        # Setting SYSTEMIMAGE_PYCURL envar to anything else forces the udm
+        # downloader.
+        with reset_envar('SYSTEMIMAGE_PYCURL'):
+            os.environ['SYSTEMIMAGE_PYCURL'] = '0'
+            self.assertIsInstance(get_download_manager(), UDMDownloadManager)
+        with reset_envar('SYSTEMIMAGE_PYCURL'):
+            os.environ['SYSTEMIMAGE_PYCURL'] = 'false'
+            self.assertIsInstance(get_download_manager(), UDMDownloadManager)
+        with reset_envar('SYSTEMIMAGE_PYCURL'):
+            os.environ['SYSTEMIMAGE_PYCURL'] = 'nope'
+            self.assertIsInstance(get_download_manager(), UDMDownloadManager)
+
+    def test_auto_detect_udm(self):
+        # If the environment variable is not set, we do auto-detection.  For
+        # backward compatibility, if udm is available on the system bus, we
+        # use it.
+        with reset_envar('SYSTEMIMAGE_PYCURL'):
+            if 'SYSTEMIMAGE_PYCURL' in os.environ:
+                del os.environ['SYSTEMIMAGE_PYCURL']
+            with patch('dbus.SystemBus.get_object') as mock:
+                self.assertIsInstance(
+                    get_download_manager(), UDMDownloadManager)
+            mock.assert_called_once_with(DOWNLOADER_INTERFACE, '/')
+
+    def test_auto_detect_curl(self):
+        # If the environment variable is not set, we do auto-detection.  If udm
+        # is not available on the system bus, we use the cURL downloader.
+        import systemimage.download
+        with ExitStack() as resources:
+            resources.enter_context(reset_envar('SYSTEMIMAGE_PYCURL'))
+            if 'SYSTEMIMAGE_PYCURL' in os.environ:
+                del os.environ['SYSTEMIMAGE_PYCURL']
+            mock = resources.enter_context(
+                patch('dbus.SystemBus.get_object', side_effect=DBusException))
+            resources.enter_context(
+                patch.object(systemimage.download, 'pycurl', object()))
+            self.assertIsInstance(
+                get_download_manager(), CurlDownloadManager)
+            mock.assert_called_once_with(DOWNLOADER_INTERFACE, '/')
+
+    def test_auto_detect_none_available(self):
+        # Again, we're auto-detecting, but in this case, we have neither udm
+        # nor pycurl available.
+        import systemimage.download
+        with ExitStack() as resources:
+            resources.enter_context(reset_envar('SYSTEMIMAGE_PYCURL'))
+            if 'SYSTEMIMAGE_PYCURL' in os.environ:
+                del os.environ['SYSTEMIMAGE_PYCURL']
+            mock = resources.enter_context(
+                patch('dbus.SystemBus.get_object', side_effect=DBusException))
+            resources.enter_context(
+                patch.object(systemimage.download, 'pycurl', None))
+            self.assertRaises(ImportError, get_download_manager)
+            mock.assert_called_once_with(DOWNLOADER_INTERFACE, '/')
