@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2014 Canonical Ltd.
+# Copyright (C) 2013-2015 Canonical Ltd.
 # Author: Barry Warsaw <barry@ubuntu.com>
 
 # This program is free software: you can redistribute it and/or modify
@@ -25,7 +25,6 @@ import unittest
 
 from contextlib import ExitStack
 from pathlib import Path
-from systemimage.config import Configuration
 from systemimage.helpers import temporary_directory
 from systemimage.settings import Settings
 from systemimage.testing.helpers import chmod, configuration
@@ -34,8 +33,7 @@ from unittest.mock import patch
 
 class TestSettings(unittest.TestCase):
     @configuration
-    def test_creation(self, ini_file):
-        config = Configuration(ini_file)
+    def test_creation(self, config):
         self.assertFalse(os.path.exists(config.system.settings_db))
         settings = Settings()
         self.assertTrue(os.path.exists(config.system.settings_db))
@@ -104,13 +102,13 @@ class TestSettings(unittest.TestCase):
         keyval.sort()
         self.assertEqual(keyval, [('a', 'ant'), ('b', 'bee'), ('c', 'cat')])
 
+    @unittest.skipIf(os.getuid() == 0, 'Test cannot succeed when run as root')
     @configuration
-    def test_settings_db_permission_denied(self, ini_file):
+    def test_settings_db_permission_denied(self, config):
         # LP: #1349478 - some tests are run as non-root, meaning they don't
         # have write permission to /var/lib/system-image.  This is where
         # settings.db gets created, but if the process can't create files
         # there, we get a sqlite3 exception.
-        config = Configuration(ini_file)
         db_file = Path(config.system.settings_db)
         self.assertFalse(db_file.exists())
         with ExitStack() as resources:
